%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% FI:PA171 - Digital Filters
%
% topic: Fourier transform and its properties, Fast Fourier transform
%
% author: David Svoboda <svoboda@fi.muni.cz>
%         Masaryk University (c) 2019
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--------------------------------------------------------------------------
% Task: show that any discrete function can be decomposed (in real domain)
% into the combination of several (appropriately shifted, scaled and weighted) 
% cosine functions.
%
% Hint: Use inverse FT and eliminate imaginary parts of complex numbers.
%--------------------------------------------------------------------------

%----------------------------------------------
% Fourier transform via matrix multiplication:
%----------------------------------------------

% clear the buffers
clear;

% Use function 'phi(j,N)' for the generation of the basis functions of DFT
% N ... length of the signal
% j ... j-th basis function, where j={0,...,N-1}

% Pay attention! The operator >>'<< and function >>transpose<< do
% not perform the same thing. Function >>transpose<< only transposes
% the vector or matrix while >>'<< also conjugates it!

% Choose suitable N - try various (reasonably small) values.
% (here we selected N = 5)
%
% Inspect how the basis appears:
% phi(0,N)
% phi(1,N)
% ...
% phi(N-1,N)

phi(0,5)
phi(1,5)
phi(2,5)
phi(3,5)
phi(4,5)

% Make notice that basis 'phi(*,N)' is orthonormal, i.e.:
% 1. phi(i,N)*phi(j,N)' == 0  if i!=j
% 2. phi(i,N)*phi(i,N)' == 1  for all 'i'
%
% Note: You can simply remove the normalization factor by editing the file
% 'phi.m'.A
%
% some examples:

phi(1,5) * phi(3,5)'
phi(4,5) * phi(2,5)'
phi(0,5) * phi(0,5)'
phi(3,5) * phi(3,5)'

% see the relationship between the couples:
% phi(1,N) ~ phi(N-1,N)
% phi(2,N) ~ phi(N-2,N)
% ...
% an example:
transpose(phi(1,5))
transpose(phi(4,5))

% Let us form a transformation matrix which rows are composed of particular
% (conjugated) basis functions:
A = [conj(phi(0,5)); conj(phi(1,5)); conj(phi(2,5)); conj(phi(3,5)); conj(phi(4,5))]

% here is a way how to create a general matrix A for any N:
%
% clear A
% for row=0:N-1
%    A(row+1,:) = conj(phi(row,N));
% end

% Generate some signal which length equals to 'N':
f = [3 2 3 3 4]

% transform the signal f: A*f'
F = A*f'

% inverse transform
B = [phi(0,5); phi(1,5); phi(2,5); phi(3,5); phi(4,5)]
% compare >>B<< and >>A'<<
A'*F
B*F

% compare the result with (normalized) fft output
result = transpose(fft(f));
normalized_result = 1/sqrt(5) * result

% see the relationship between:
% (F is indexed from '1')
% F(2) ~ F(N)
% F(3) ~ F(N-1)
% ...
% an example:
F(2)
F(5)

% show real part, imaginary part, magnitude, and phase
real(F)
imag(F)
abs(F)
angle(F)

% observe the symmetry of the transformed signal
zero_left_aligned = F
zero_centered = fftshift(F)

%-----------------------------------------------------------------
% Task to solve:
% Open 'my_fft.m' file and finish the missing parts.
%-----------------------------------------------------------------
